/*
This file is part of wmfreqview, a wmaker dock app that shows cpufreq over time.
Copyright (C) 2005 John Tsiombikas

wmfreqview is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

wmfreqview is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with wmfreqview; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <alloca.h>
#include <unistd.h>
#include <X11/X.h>
#include <X11/xpm.h>
#include <X11/Xlib.h>
#include "wmgeneral.h"
#include "wmfreqview.h"
#include "pixmaps.h"

#define MIN_FNAME	"scaling_min_freq"
#define MAX_FNAME	"scaling_max_freq"
#define FREQ_FNAME	"scaling_cur_freq"

int get_freq_limits(int *min, int *max);
int get_freq(void);

char *freq_path = "/sys/devices/system/cpu/cpu0/cpufreq/";
char *cur_freq_fname;

int min, max;
int freq_hist[GRAPH_COLS];
int top;

unsigned long update_interval = 500000;

const char usage[] = "wmfreqview options:\n"
	"   -t\tpoll interval in milliseconds (def: 500)\n"
	"   -p\tpath to the cpufreq sysfs directory\n"
	"   -h\tthis usage information\n";

int main(int argc, char **argv) {
	int i;
	XEvent	event;

	for(i=1; i<argc; i++) {
		if(argv[i][0] == '-' && argv[i][2] == 0) {
			switch(argv[i][1]) {
			case 't':
				if(isdigit(argv[++i][0])) {
					update_interval = atol(argv[i]) * 1000;
				} else {
					puts(usage);
					return 0;
				}
				break;

			case 'p':
				{
					unsigned int len = strlen(argv[++i]);
					
					freq_path = malloc(len + 2);
					strcpy(freq_path, argv[i]);
					
					if(freq_path[len - 1] != '/') {
						freq_path[len++] = '/';
						freq_path[len] = 0;
					}
				}						
				break;

			case 'h':
			default:
				puts(usage);
				return 0;
			}
		} else {
			puts(usage);
			return 0;
		}
	}			

	if(get_freq_limits(&min, &max) == -1) {
		fprintf(stderr, "could not retrieve CPU scaling limits, no cpufreq support?\n");
		return -1;
	}

	cur_freq_fname = malloc(strlen(freq_path) + strlen(FREQ_FNAME) + 1);
	strcpy(cur_freq_fname, freq_path);
	strcat(cur_freq_fname, FREQ_FNAME);

	memset(freq_hist, 0, sizeof freq_hist);
	top = 0;

	open_window(argc, argv, xpm_master, xpm_mask_bits, xpm_mask_width, xpm_mask_height);

	XSetForeground(display, NormalGC, GetColor("rgb:00/ff/00"));
	XSetLineAttributes(display, NormalGC, 2, LineSolid, CapRound, JoinRound);
	
	for(;;) {
		freq_hist[top] = get_freq();
		top = (top + 1) % GRAPH_COLS;
		
		RedrawWindow();

		/* Process any pending X events. */
		while(XPending(display)) {
			XNextEvent(display, &event);
			
			switch (event.type) {
			case Expose:
				RedrawWindow();
				break;
				
			default:
				break;
			}
		}

		usleep(update_interval);
	}

	return 0;
}

#define MAX(a, b)	((a) > (b) ? (a) : (b))

int get_freq_limits(int *min, int *max) {
	FILE *fp;
	char *fname;
	char buf[32];

	fname = alloca(strlen(freq_path) + MAX(strlen(MIN_FNAME), strlen(MAX_FNAME)) + 1);
	strcpy(fname, freq_path);
	strcat(fname, MIN_FNAME);

	if(!(fp = fopen(fname, "r"))) {
		return -1;
	}
	
	fgets(buf, 32, fp);
	*min = atoi(buf);
	fclose(fp);

	strcpy(fname, freq_path);
	strcat(fname, MAX_FNAME);

	if(!(fp = fopen(fname, "r"))) {
		return -1;
	}
	
	fgets(buf, 32, fp);
	*max = atoi(buf);
	fclose(fp);

	return 0;
}

int get_freq(void) {
	char buf[32];
	FILE *freq_file;
	
	if(!(freq_file = fopen(cur_freq_fname, "r"))) {
		fprintf(stderr, "could not retrieve current CPU frequency, no cpufreq support?\n");
		return -1;
	}
	
	fseek(freq_file, 0, SEEK_SET);
	fgets(buf, 32, freq_file);

	fclose(freq_file);
	return atoi(buf);
}
